var BodyParser = require( 'body-parser' );
var http = require('http');
var express = require('express');
var decode = require('./decode.js')
var mqtt = require("mqtt");
var dotenv = require('dotenv');
var fs = require('fs-extra');

dotenv.config();

var port = process.env.PORT;
var serverhost   = process.env.MQTT_HOST;
var serverport   = process.env.MQTT_PORT;
var filename     = process.env.FILE;

var pendingEvents = [];
//===========================================================================
var app = express();
app.use( BodyParser.urlencoded( { extended: false } ) );
app.use( BodyParser.json() );

function doSend(packet) {
  if (packet.data != undefined) {
		var client = mqtt.connect(serverhost, port=serverport);
		client.on("connect", function() {
			console.log("Connected: " + packet.device_name + " " + JSON.stringify(packet.data));
			client.publish("shio/" + packet.device_name, JSON.stringify(packet.data));
			client.end();
		});
		client.on("error", function(err) {
			console.log(err);
			client.end();
		})
  }
}

var server = http.createServer(app);
if (filename != undefined) {
  var wstream = fs.createWriteStream(filename, {'flags': 'w'});
  wstream.on('finish', function () {
  });
  wstream.write('timestamp,');
  wstream.write('DP,');
  wstream.write('rssi,');
  wstream.write('deviceId\n');
  wstream.end();
}
//===========================================================================
app.post('/',function(request, response) {
	var device_name = request.param('deviceId');
	var payload = request.param('payload');
  var data = decode.DecodePayload(decode.hexToBytes(payload));
  response.writeHead(200, {"Content-Type": "text/plain"});
	response.write("Success");
	response.end();
  console.log('Received packet:');
	console.log(JSON.stringify(request.body));
	console.log('Sensor data:');
	console.log(JSON.stringify(data));

  // Send to MQTT
	if ((serverhost != undefined) && (serverport != undefined)) {
		var packet = new Object();
		packet.device_name = device_name;
		packet.data = data;
		pendingEvents.push(packet);
	}

  // Write to a file
  if (filename != undefined) {
    if (data.ibeacon_num != undefined) {
      var wstream = fs.createWriteStream(filename, {'flags': 'a'});
      if (data.ibeacon_num == 0) {
        wstream.write(request.body.timestamp + ',');
        wstream.write('-,');
        wstream.write('-,');
        wstream.write(request.body.deviceId + '\n');
      } else {
        for (var i = 0; i < data.ibeacon_num; i++) {
          wstream.write(request.body.timestamp + ',');
          wstream.write(data.ibeacon_set[i][0] + ',');
          wstream.write(data.ibeacon_set[i][1] + ',');
          wstream.write(request.body.deviceId + '\n');
        }
      }
      wstream.end();
    }
  }
});

///===========================================================================
server.listen(port,function(){
    console.log('Server start! (' + port + ')');
		if ((serverhost != undefined) && (serverport != undefined)) {
			console.log('To MQTT server:' + serverhost + ":" + serverport);
		}
});

setInterval(() => {
  while (pendingEvents.length > 0) {
    const toSendEvents = pendingEvents.shift();
    doSend(toSendEvents);
  }
}, 1000);
